/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 253 (Table 5.3 & Figure 5.3)
**
*/

new;
library rpb,pgraph;

cls;

datFileName = "data\\commodities";

data = loadd(datFileName);
vnames = "" $+ getname(datFileName);
loads strData = ^datFileName;

let string Selected_Commodities = "CL" "CO" "NG" "HO" "W" "C" "S" "CT" "HG" "GC" "SI" "PL" "FEDL01";

nCommodities = rows(Selected_Commodities) - 1;

Dates = Data[.,1];
nDates = rows(Dates);
indx = indsav(Selected_Commodities,vnames);
Data = Data[.,indx];
Data = Fill_Miss(Data,4);

Basket = Data[.,1:nCommodities];
Libor = Data[.,nCommodities+1]/100;
Libor_Index = Capitalized_Libor(Dates,Libor);
nBasket = cols(Basket);

Months = trunc(Dates/100);
RD_Monthly = Months ./= lag1(Months);
RD_Monthly[1] = 1;

x_EW = zeros(nDates,nBasket);
x_ERC = zeros(nDates,nBasket);
rc_EW = zeros(nDates,nBasket);
rc_ERC = zeros(nDates,nBasket);

{cov,vol,cor} = Rolling_Covariance(Dates,Basket,260,2);

t = 1;
do until t > nDates;

  if RD_Monthly[t] == 0;

    x_EW[t,.] = x_EW[t-1,.];
    x_ERC[t,.] = x_ERC[t-1,.];
    rc_EW[t,.] = rc_EW[t-1,.];
    rc_ERC[t,.] = rc_ERC[t-1,.];

  else;

    cls;
    print /flush ftosa(t,"Iteration = %lf",3,2);

    cov_t = arraytomat(cov[t,.,.]);
    indx = selif(seqa(1,1,nBasket),sumr(missrv(cov_t,0)) ./= 0);
    nValid = rows(indx);
    cov_t = cov_t[indx,indx];
  
    x = ones(nValid,1)/nValid;
    {vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov_t);
    x_EW[t,indx] = x';
    rc_EW[t,indx] = prc';

    stderr = sqrt(diag(cov_t));
    x0 = 1 ./ stderr;
    x0 = x0 / sumc(x0);
    {x,vol,mr,rc,prc} = Compute_ERC_Portfolio(x0,cov_t);
    x_ERC[t,indx] = x';
    rc_ERC[t,indx] = prc';

  endif;

  t = t + 1;
endo;

cls;

F_EW = Simulate_Monthly_Backtest(Dates,ones(nDates,1),Libor_Index,x_EW,Basket);
F_ERC = Simulate_Monthly_Backtest(Dates,ones(nDates,1),Libor_Index,x_ERC,Basket);

Funds = F_EW ~ F_ERC;

Begin_Date = 19900101;

Data = Dates~Funds~Libor_Index;
Data = selif(Data,Data[.,1] .>= Begin_Date);
stats = zeros(6,2);

iter = 1;
do until iter > 2;
  report = Reporting_Backtest(Data[.,1],Data[.,1+iter],0,Data[.,4],0,0);
  stats[1,iter] = vread(Report,"mu");
  stats[2,iter] = vread(Report,"sigma");
  stats[3,iter] = vread(Report,"SR");
  stats[4,iter] = vread(Report,"MDD");
  stats[5,iter] = vread(Report,"sk");
  stats[6,iter] = vread(Report,"kt")-3.0;
  iter = iter + 1;
endo;

stats[1:2 4,.] = stats[1:2 4,.] * 100;
print ftosa(stats',"%lf",3,2);

indx_t = seqa(1,1,nDates);
indx_t = selif(indx_t,(Dates .>= Begin_Date) .and RD_Monthly);

Dates = Dates[indx_t];
Funds = Funds[indx_t,.];
RC_EW = RC_EW[indx_t,.];
x_ERC = x_ERC[indx_t,.];

Funds = 100 * Funds ./ Funds[1,.];

graphset;
  window3d;

  _pdate = ""; _pnum = 2; _pframe = 0;
  _plwidth = 10;
  _paxht = 0.20; _pnumht = 0.20; _ptitlht = 0.275;

  _pboxctl = {.25 0 3 10 90};
  _pcolor = seqa(1,1,9)|12;

  ytics(0,25,5,2);

  setwind(1);     
    title("\216RC of the EW portfolio");
    call box(0,100*rc_EW);

  setwind(2);     
    title("\216Weights of the ERC portfolio");
    call box(0,100*x_ERC);

  setwind(3);
    _pltype = 6|3; _plwidth = 10; _pcolor = 2|1;
    title("\216Simulated performance");
    xtics(1,rows(Funds),12*5,2);
    let xlab = "1990" "95" "2000" "05" "10" "15";
    asclabel(xlab,0);
    ytics(0,800,100,2);
    _plegstr = "EW\000ERC";
    _plegctl = {2 5 1.75 4};
    xy(0,Funds);

    graphprt("-c=1 -cf=chap5-1-2-2.eps");

  endwind;